"use strict";
/* This is a conversion from BLAS to Typescript/Javascript
Copyright (C) 2018  Jacob K.F. Bogers  info@mail.jacob-bogers.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.div_rxr = exports.mul_cxc = exports.mul_cxr = exports.mul_rxr = exports.mul_rxc = exports.possibleScalar = exports.lowerChar = exports.errWrongArg = exports.errMissingIm = exports.errorMsg = exports.ERROR = exports.isOne = exports.isZeroE = exports.isZero = void 0;
//intrinsic routines of fortran
function isZero(v) {
    return v.im === 0 && v.re === 0;
}
exports.isZero = isZero;
function isZeroE(re, im) {
    return re === 0 && im === 0;
}
exports.isZeroE = isZeroE;
function isOne(v) {
    return v.re === 1 && v.im === 0;
}
exports.isOne = isOne;
var ERROR;
(function (ERROR) {
    ERROR[ERROR["ERR_MISSING_IMAGINARY"] = 1] = "ERR_MISSING_IMAGINARY";
    ERROR[ERROR["ERR_MISSING_REAL"] = 2] = "ERR_MISSING_REAL";
    ERROR[ERROR["ERR_WRONG_ARGUMENT"] = 3] = "ERR_WRONG_ARGUMENT";
})(ERROR = exports.ERROR || (exports.ERROR = {}));
// private
const errors = new Map([
    [ERROR.ERR_MISSING_IMAGINARY, 'Missing imaginary part for %s'],
    [ERROR.ERR_MISSING_REAL, 'Missing real part for %s'],
    [ERROR.ERR_WRONG_ARGUMENT, 'function:[%s], argument [%s] is has invalid value.'],
]);
const ERROR_UKNOWN = 'Unkown Error code used! [%s]';
function errorMsg(errNo, ...fmt) {
    const msg = errors.get(errNo) || ERROR_UKNOWN;
    return fmt.reduce((p, v) => p.replace('%s', v), msg);
}
exports.errorMsg = errorMsg;
//helpers for errorMsg
function errMissingIm(fmt) {
    return errorMsg(ERROR.ERR_MISSING_IMAGINARY, fmt);
}
exports.errMissingIm = errMissingIm;
function errWrongArg(arg, value) {
    return errorMsg(ERROR.ERR_WRONG_ARGUMENT, arg, JSON.stringify(value));
}
exports.errWrongArg = errWrongArg;
function lowerChar(c) {
    if (typeof c !== 'string') {
        return '';
    }
    const cc = c.charCodeAt(0);
    if ((cc >= 65 && cc <= 90) || (cc >= 97 && cc <= 122)) {
        return String.fromCharCode(cc | 0x20);
    }
    return c;
}
exports.lowerChar = lowerChar;
function possibleScalar(x) {
    return x.length === 1 ? x[0] : x;
}
exports.possibleScalar = possibleScalar;
function mul_rxc(re1, im1, c) {
    const re = re1 * c.re - im1 * c.im;
    const im = re1 * c.im + im1 * c.re;
    return { re, im };
}
exports.mul_rxc = mul_rxc;
function mul_rxr(re1, im1, re2, im2) {
    const re = re1 * re2 - im1 * im2;
    const im = re1 * im2 + im1 * re2;
    return { re, im };
}
exports.mul_rxr = mul_rxr;
function mul_cxr(c, re1, im1) {
    const re = c.re * re1 - c.im * im1;
    const im = c.re * im1 + c.im * re1;
    return { re, im };
}
exports.mul_cxr = mul_cxr;
function mul_cxc(c1, c2) {
    const re = c1.re * c2.re - c1.im * c2.im;
    const im = c1.re * c2.im + c1.im * c2.re;
    return { re, im };
}
exports.mul_cxc = mul_cxc;
function div_rxr(re1, im1, re2, im2) {
    // (a+ib)/c+id) = (ac+bd)/(c*c +d*d) + i(-ad+bc)/(c*c+d*d)
    const norm = re2 * re2 + im2 * im2;
    const re = (re1 * re2 + im1 * im2) / norm;
    const im = (-re1 * im2 + im1 * re2) / norm;
    return { re, im };
}
exports.div_rxr = div_rxr;
//# sourceMappingURL=f_func.js.map