"use strict";
/* This is a conversion from BLAS to Typescript/Javascript
Copyright (C) 2018  Jacob K.F. Bogers  info@mail.jacob-bogers.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.ctbsv = void 0;
const f_func_1 = require("../../../f_func");
const norm_lower_1 = require("./norm-lower");
const norm_upper_1 = require("./norm-upper");
const trans_lower_1 = require("./trans-lower");
const trans_upper_1 = require("./trans-upper");
function ctbsv(uplo, trans, diag, n, k, a, lda, x, incx) {
    if (x.i === undefined) {
        throw new Error((0, f_func_1.errMissingIm)('x.i'));
    }
    if (a.i === undefined) {
        throw new Error((0, f_func_1.errMissingIm)('a.i'));
    }
    // faster then String.toLowerCase()
    const ul = (0, f_func_1.lowerChar)(uplo);
    const tr = (0, f_func_1.lowerChar)(trans);
    const dg = (0, f_func_1.lowerChar)(diag);
    let info = 0;
    if (!'ul'.includes(ul)) {
        info = 1;
    }
    else if (!'ntc'.includes(tr)) {
        info = 2;
    }
    else if (!'un'.includes(dg)) {
        info = 3;
    }
    else if (n < 0) {
        info = 4;
    }
    else if (k < 0) {
        info = 5;
    }
    else if (lda < k + 1) {
        info = 7;
    }
    else if (incx === 0) {
        info = 9;
    }
    if (info !== 0) {
        throw new Error((0, f_func_1.errWrongArg)('ctbsv', info));
    }
    //*     Quick return if possible.
    if (n === 0)
        return;
    const noconj = tr === 't';
    const nounit = dg === 'n';
    const kx = incx < 0 ? 1 - (n - 1) * incx : 1;
    let func;
    if (tr === 'n') {
        if (ul === 'u') {
            func = norm_upper_1.normUpper;
        }
        else {
            func = norm_lower_1.normLower;
        }
    }
    else {
        if (ul === 'u') {
            func = trans_upper_1.transUpper;
        }
        else {
            func = trans_lower_1.transLower;
        }
    }
    func(kx, x, incx, a, noconj, nounit, n, k);
}
exports.ctbsv = ctbsv;
//# sourceMappingURL=index.js.map