export declare function isZero(v: Complex): boolean;
export declare function isZeroE(re: number, im: number): boolean;
export declare function isOne(v: Complex): boolean;
export declare type Complex = {
    re: number;
    im: number;
};
export declare type fpArray = Float32Array | Float64Array;
export declare type FortranSetterGetter = (index: number) => (re?: number, im?: number) => number | Complex;
export declare type FortranArr = {
    base: number;
    r: fpArray;
    i?: fpArray;
    s: FortranSetterGetter;
    toArr: () => Complex[] | number[];
};
export declare type FortranArrEComplex = {
    base: number;
    r: fpArray;
    i: fpArray;
    s: FortranSetterGetter;
    toArr: () => Complex[] | number[];
};
export declare enum ERROR {
    ERR_MISSING_IMAGINARY = 1,
    ERR_MISSING_REAL = 2,
    ERR_WRONG_ARGUMENT = 3
}
export declare function errorMsg(errNo: ERROR, ...fmt: string[]): string;
export declare function errMissingIm(fmt: string): string;
export declare function errWrongArg(arg: string, value: any): string;
export declare type FortranMatrixSetterGetter = (colSize: number) => (nrCols: number) => number | Complex;
export declare type MatrixType = 'n' | 'b' | 'bu' | 'bl' | 'pl' | 'pu';
export interface Matrix {
    readonly rowBase: number;
    readonly colBase: number;
    readonly nrCols: number;
    readonly nrRows: number;
    readonly r: fpArray;
    readonly i?: fpArray;
    colOfEx(n: number): number;
    coord(col: number): (a: number) => number;
    setCol(col: number, rowStart: number, rowEnd: number, value: number): void;
    slice_used_for_test(rowStart: number, rowEnd: number, colStart: number, colEnd: number): Matrix;
    slice(rowStart: number, rowEnd: number, colStart: number, colEnd: number): Matrix;
    setLower(value?: number): Matrix;
    setUpper(value?: number): Matrix;
    upperBand(value: number): Matrix;
    lowerBand(value: number): Matrix;
    packedUpper(value?: number): FortranArr;
    packedLower(value?: number): FortranArr;
    real(): Matrix;
    imaginary(): Matrix;
    toArr(): Complex[] | number[];
}
export interface MatrixEComplex extends Matrix {
    readonly i: fpArray;
}
export declare function lowerChar<T extends string>(c?: T): T;
export declare function possibleScalar<T>(x: T[]): T | T[];
export declare function mul_rxc(re1: number, im1: number, c: Complex): Complex;
export declare function mul_rxr(re1: number, im1: number, re2: number, im2: number): Complex;
export declare function mul_cxr(c: Complex, re1: number, im1: number): Complex;
export declare function mul_cxc(c1: Complex, c2: Complex): Complex;
export declare function div_rxr(re1: number, im1: number, re2: number, im2: number): Complex;
//# sourceMappingURL=f_func.d.ts.map